import obj_player
import math,random

class Enemy:
    '''This is the enemy template and won't be used in game except for actual enemies to inherit from'''
    def __init__(self,x,y):
        self.x = x
        self.y = y
        self.lifetime = 0
        self.health = 1
        self.output = []
        self.miss = 0
        self.hitbox = 10
        self.base_score = 0
    
    def update(self):
        self.lifetime += 1

    def check_mortality(self):
        '''This function: destroys the enemy if it goes far out of bounds'''
        #enemies don't kill the player on touch: this is intentional
        if self.x < -50 or self.x > 650 or self.y < -50 or self.y > 450:
            self.health = 0
            self.base_score = 0
        if self.x > obj_player.x + 15 and self.y < obj_player.y + 3 + self.hitbox and self.y > obj_player.y - 3 - self.hitbox and obj_player.beam_frame == 4:
            self.health += -1
        pass



class Basic(Enemy):
    '''The first enemy that will appear in the game.'''
    def __init__(self,x,y):
        Enemy.__init__(self,x,y)
        self.base_score = 100
        self.health = 12

    def update(self):
        '''By checking the value of lifetime, the enemy can create new bullets a certain amount of time after creation'''
        self.output = []
        self.lifetime += 1
        self.x -= 1
        if (self.lifetime + 40) % 60 == 0:
            self.output = [['Straight',str(self.x),str(self.y),str(160),str(2)],['Straight',str(self.x),str(self.y),str(180),str(2)],['Straight',str(self.x),str(self.y),str(200),str(2)]]
        self.check_mortality()

class Explosion(Enemy):
    '''Explosions can just be technically an enemy since this class has everything needed'''
    def __init__(self, x, y):
        super().__init__(x, y)
        self.lifetime = 0
    
    def update(self):
        '''
        Updates the explosion at half speed so the animation will play at 30fps instead of 62.5 or whatever.
        The explosion does not move
        '''
        self.lifetime += 0.5
        self.check_mortality()

    def check_mortality(self):
        '''Dies when lifetime is 7 (there is no frame 7)'''
        if self.lifetime == 7:
            self.health = 0

class Fast(Enemy):
    def __init__(self, x, y):
        super().__init__(x, y)
        self.base_score = 150
        self.health = 8
        self.offset_y = y

    def update(self):
        '''By checking the value of lifetime, the enemy can create new bullets a certain amount of time after creation'''
        self.output = []
        self.lifetime += 1
        self.x -= 2
        self.y = self.offset_y + (math.sin(self.x/50) * 50)

        if (self.lifetime + 100) % 120 == 0:
            self.output = [['DelayHoming',str(self.x),str(self.y),str(160),str(3)],['DelayHoming',str(self.x),str(self.y),str(200),str(3)],['DelayHoming',str(self.x),str(self.y),str(220),str(3)],['DelayHoming',str(self.x),str(self.y),str(140),str(3)]]
        self.check_mortality()

class Complex(Enemy):
    def __init__(self, x, y):
        super().__init__(x, y)
        self.base_score = 300
        self.health = 15

    def update(self):
        self.output = []
        self.lifetime += 1
        self.x -= 0.5 + self.lifetime / 100

        if (self.lifetime) % 1 == 0: #always do this
            self.output = [['Straight',str(self.x),str(self.y),str(self.lifetime * 11),str(3)]]
        self.check_mortality()

class Boss(Enemy):
    def __init__(self, x, y):
        super().__init__(x, y)
        self.health = 500
        self.base_score = 5000
        self.x = 450

    def update(self):
        self.output = []
        self.lifetime += 1
        if self.lifetime <= 150:
            self.x += -1
        elif self.lifetime >= 777:
            if self.lifetime % 70 == 0:
                for yout in range (0,401,30):
                    self.output.append(['Straight',str(605),str(yout),str(180),str(2)])
            if self.lifetime % 70 == 35:
                for yout in range (15,401,30):
                    self.output.append(['Straight',str(605),str(yout),str(180),str(2)])
            if self.lifetime % 100 == 0:
                for _ in range(3):
                    self.output.append(['DelayHoming',str(605),str(random.randint(0,400)),str(180),str(3)])
            self.check_mortality()

    def check_mortality(self):
        '''This function: destroys the enemy if it goes far out of bounds'''
        #enemies don't kill the player on touch: this is intentional
        if obj_player.beam_frame == 4: #don't even bother to check the location it's the entire screen
            self.health += -1
        if self.health <= 0:
            obj_player.life = 0
